#!/bin/bash

# Copyright (c) 2021, Habana Labs Ltd.  All rights reserved.
# Copyright (c) 2019 NVIDIA CORPORATION. All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

to_download=${1:-"wiki_only"}

#Download
data_dir=$(pwd)
BERT_PREP_WORKING_DIR="${data_dir}"
export BERT_PREP_WORKING_DIR="${BERT_PREP_WORKING_DIR}"

# Download Wikipedia dataset and/or Bookscorpus dataset
echo "Download dataset ${to_download}"

if [ "$to_download" = "wiki_only" ] ; then
    DATASET="wikicorpus_en"

    # checkout WikiExtractor scripts
    echo "Checkout WikiExtractor repository"
    git clone https://github.com/attardi/wikiextractor.git && cd wikiextractor && git checkout 6408a430fc504a38b04d37ce5e7fc740191dee16 && cd ..
elif [ "$to_download" = "wiki_books" ] ; then
    DATASET="books_wiki_en_corpus"

    # checkout BookCorpus download scripts
    git clone https://github.com/soskek/bookcorpus.git
elif [ "$to_download" = "wikitext-2" ] ; then
    DATASET="wikitext-2"
fi

echo "Download pretrained weights for bert_large_uncased"
echo "${data_dir}"
$PYTHON ${data_dir}/bertPrep.py --action download --dataset google_pretrained_weights_bert_large_uncased  # Includes vocab

$PYTHON ${data_dir}/bertPrep.py --action download --dataset $DATASET
$PYTHON ${data_dir}/bertPrep.py --action text_formatting --dataset $DATASET
$PYTHON ${data_dir}/bertPrep.py --action sharding --dataset $DATASET

# Create HDF5 files Phase 1
$PYTHON ${data_dir}/bertPrep.py --action create_hdf5_files --dataset $DATASET --max_seq_length 128 \
--max_predictions_per_seq 20 --vocab_file $BERT_PREP_WORKING_DIR/download/google_pretrained_weights/uncased_L-24_H-1024_A-16/vocab.txt --do_lower_case 1

# Create HDF5 files Phase 2
$PYTHON ${data_dir}/bertPrep.py --action create_hdf5_files --dataset $DATASET --max_seq_length 512 \
--max_predictions_per_seq 80 --vocab_file $BERT_PREP_WORKING_DIR/download/google_pretrained_weights/uncased_L-24_H-1024_A-16/vocab.txt --do_lower_case 1
